/***************************************************************************
 *   Copyright (C) 2008 by Dirk Hartmann                                   *
 *   2monex@gmx.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PACKAGEMETADATA_H
#define PACKAGEMETADATA_H

#include "rdnamespace.h"

#include <QString>
#include <QStringList>
#include <QMap>
#include <QHash>

struct PackageVersionAndName {
	QString packageName;
	QString packageVersion;
};

class PackageMetaData
{
	public:
		PackageMetaData();
		PackageMetaData(const QString &fileName, const QString &packageName, const QString &version, const quint64 size, const QString &architecture, const QString &shaCheckSum);
		PackageMetaData(const QString &fileName, const QString &architecture); // tries to extract informations from the filename
		
		static PackageVersionAndName extractVersionAndName(const QString &fileName, const QString &archString);
		static QStringList archStringList(const RPM::Architectures archs); // generates a string list from the given archs
		static bool versionIsGreaterThan(const QString &base, const QString &other);
		
		bool operator>(const PackageMetaData &rhs) const;
		bool operator<(const PackageMetaData &rhs) const;
		
		void clear();
		void setFile(const QString &fileName) {file = fileName;}
		void setPackageName(const QString &pName) {name = pName;}
		void setSize(const quint64 &size) {fileSize = size;}
		void setSize(const QString &size);
		void addRequire(const QString &requirement);
		void addProvide(const QString &provide);
		void setArch(const QString &arch) {this->arch = arch;};
		void setVersion(const QString &version) {packageVersion = version;}
		void setShaCheckSum(const QString &cSum) {checkSum = cSum;}
		
		QString packageName() const {return name;}
		QString fileName() const {return file;}
		QString architecture() const {return arch;}
		QStringList provides() const {return packageProvides;}
		QStringList requires() const {return packageRequires;}
		QString version() const {return packageVersion;}
		QString shaCheckSum() const {return checkSum;}
		quint64 size() const {return fileSize;}
		QString sizeAsString() const;
	
	private:
		static QStringList fixRpmVersionPart(const QStringList &versionParts);
		
		QString file;
		QString name;
		QString arch;
		QString packageVersion;
		quint64 fileSize;
		QStringList packageRequires;
		QStringList packageProvides;
		QString checkSum; // SHA check sum only for remote meta datas

};

namespace MetaData {
	typedef QHash<QString, PackageMetaData> PackageMetaDatas; // only for one architecture <package name, meta datas>
	typedef QMap<QString, PackageMetaDatas> ArchsMetaDatas; // for all archs
}

#endif
