/***************************************************************************
 *   Copyright (C) 2008 by Dirk Hartmann                                   *
 *   2monex@gmx.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RPMDOWNLOADERSETTINGS_H
#define RPMDOWNLOADERSETTINGS_H

#include <QDir>
#include <QString>

class RpmDownloaderSettings
{
	public:
		RpmDownloaderSettings();

		~RpmDownloaderSettings();

		void setUpdateInterval(int interval);
		int updateInterval() const {return rpmUpdateInterval;}

		void setDeleteOldVersions(const bool deleteOld);
		bool deleteOldVersions() const {return deleteOldVersionsFromDisk;}

		QDir tempDir() const {return tempDirectory;}
		void setTempDir(const QDir &newDir);

		QDir cacheDir() const {return cacheDirectory;}
		void setCacheDir(const QDir &newDir);

		QString checksumCommand() const {return chcksumCommand;}
		void setChecksumCommand(const QString &newCommand);

		QString gunzipCommand() const {return g_unzipCommand;}
		void setGunzipCommand(const QString &newCommand);

		bool useMemDbSatSolving() const {return memDbSatSolve;}
		void setMemDbSatSolving(const bool use);
		
		bool doCheckSumCheckOnDownloadedPackages() const{return checkSumCheck;}
		void setDoCheckSumCheckOnDownloadedPackages(const bool doIt);


	private:
		int rpmUpdateInterval;
		bool deleteOldVersionsFromDisk;

		QDir cacheDirectory; // for yum repositories
		QDir tempDirectory; // for temporary files
		QString chcksumCommand; // for doing a checksum check of downloaded yum repo meta files
		QString g_unzipCommand; // unzip yum meta files

		// specifies if the whole database should be loaded into the memory
		// when resolving satisfactions which will increase the speed a lot
		// but consumes more memory
		bool memDbSatSolve;
		bool checkSumCheck;

};

RpmDownloaderSettings &rpmDownloaderSettings(); // satic (singleton) settings object

#endif
