/***************************************************************************
 *   Copyright (C) 2008 by Dirk Hartmann                                   *
 *   2monex@gmx.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "updaterpmstatusthread.h"

#include <QFtp>
#include <QMutex>

QMutex mutex;

UpdateRpmStatusThread::UpdateRpmStatusThread(QObject *parent)
		: QThread(parent)
{
	profilePos = -1;
	ftp = new QFtp(this);
	
	connect(ftp, SIGNAL(done(bool)), this, SLOT(ftpRequestFinished(bool)));
	connect(ftp, SIGNAL(listInfo(QUrlInfo)), this, SLOT(gotFtpListEntry(QUrlInfo)));
	connect(ftp, SIGNAL(commandStarted(int)), this, SLOT(ftpCommandStarted(int)));
	connect(ftp, SIGNAL(stateChanged(int)), this, SLOT(ftpStateChanged(int)));
}


UpdateRpmStatusThread::~UpdateRpmStatusThread()
{
}

void UpdateRpmStatusThread::setPosOfProcessedProfile(const int pos)
{
	profilePos = pos;
}

void UpdateRpmStatusThread::setRpms(const QMap<QString, Rpm> & rpms)
{
	currentProfile.setRpms(rpms);
}

void UpdateRpmStatusThread::run()
{
	qDebug("Thread started for %i", profilePos);
	QMap<QString, Rpm> rpms = currentProfile.getRpms();
	QString key;
	foreach(key, rpms.keys()) {
		rpms[key].updateLocalVersion(currentProfile.downloadDir(), currentProfile.architecures());
	}
	
	updateRpmLists();
	
	getRpmLists();
	currentProfile.setRpms(rpms);
}

void UpdateRpmStatusThread::setProfile(const RepositoryProfile & profile)
{
	currentProfile = profile;
}

void UpdateRpmStatusThread::updateRpmLists()
{
	mutex.lock(); // lock now
	rpmLists.clear();
	if (currentProfile.architecures() & RPM::i586) {
		rpmLists.insert("i586", QStringList());
	}
	if (currentProfile.architecures() & RPM::i686) {
		rpmLists.insert("i686", QStringList());
	}
	if (currentProfile.architecures() & RPM::x86_64) {
		rpmLists.insert("x86_64", QStringList());
	}
	if (currentProfile.architecures() & RPM::noarch) {
		rpmLists.insert("noarch", QStringList());
	}
	updateNextItem();
}

QMap< QString, QStringList > UpdateRpmStatusThread::getRpmLists() const
{
	mutex.lock();
	return rpmLists;
	mutex.unlock();
}

void UpdateRpmStatusThread::updateNextItem()
{
	QString key;
	bool allKeysUpdated = true;
	foreach(key, rpmLists.keys()) {
		if (rpmLists.value(key).isEmpty()) {
			allKeysUpdated = false;
			break;
		}
	}
	
	if (allKeysUpdated) {
		mutex.unlock();
		return;
	}
	
	currentArch = key;
	QUrl url(currentProfile.serverAddress());
	url.setPath(currentProfile.serverPath() + "/" + currentArch);
	qDebug("URL: %s", qPrintable(url.toString()));
	if (url.isValid()) {
		if (url.scheme() == "ftp") {
			ftp->connectToHost(url.host(), url.port(21));
			ftp->login();
			ftp->list(url.path());
			ftp->close();
		} // TODO HTTP
	} else {
		qDebug("Invalid URL: %s", qPrintable(url.toString()));
		mutex.unlock();
	}
}

void UpdateRpmStatusThread::gotFtpListEntry(QUrlInfo info)
{
	if (info.isFile()) {
		rpmLists[currentArch].push_back(info.name());
	}
}

void UpdateRpmStatusThread::ftpRequestFinished(bool success)
{
	Q_UNUSED(success);
	qDebug("ftp done");
	updateNextItem();
}

void UpdateRpmStatusThread::ftpCommandStarted(int id)
{
	qDebug("command id %i", id);
}

void UpdateRpmStatusThread::ftpStateChanged(int i)
{
	qDebug("state %i", i);
}

