/***************************************************************************
 *   Copyright (C) 2008 by Dirk Hartmann                                   *
 *   2monex@gmx.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "yumcachebuilderthread.h"

#include "yumfileconstants.h"

#include <QFile>
#include <QXmlSimpleReader>

YumCacheBuilderThread::YumCacheBuilderThread(QObject *parent)
		: QThread(parent), success(true)
{
}


YumCacheBuilderThread::~YumCacheBuilderThread()
{
	terminateCacheBuilder();
}

void YumCacheBuilderThread::setDatabase(const QString & pathToDatabase)
{
	databasePath = pathToDatabase;
}

void YumCacheBuilderThread::setPathToCacheDir(const QString & path)
{
	cache = path;
}

void YumCacheBuilderThread::setRepoName(const QString & name)
{
	repoName = name;
}

void YumCacheBuilderThread::run()
{
	// here runs the threaded part
	errMsg.clear();
	success = true;
	
	QFile file(cache + "/" + Yum::primaryFileName);
// 	QXmlInputSource inputSource(&file);
// 	QXmlSimpleReader reader;
	
	if (!reader.initDatabase(databasePath, repoName)) {
		errMsg = reader.readError();
		success = false;
		return;
	}
	
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		errMsg = tr("Cannot read file %1: %2").arg(file.fileName()).arg(file.errorString());
		success = false;
		return;
	}
	
// 	reader.setContentHandler(&handler);
// 	reader.setErrorHandler(&handler);
	success = reader.read(&file);
	errMsg = reader.readError();
}

void YumCacheBuilderThread::terminateCacheBuilder()
{
	if(isRunning())
		reader.abortUpdate();
}



